YandirsMight = {
	name = "YandirsMight",
	version = "0.1",
	varVersion = 1, -- savedVariables version
	uiLocked = true,
}

-- Giant's Endurance (non perfect / perfect)
local GE_IDS = {135950, 138013}
-- Giant's Might  (non perfect / perfect)
local GM_IDS = {135951, 138019}
local STATES = {
	[0] = { 1,   0, 0},
	[1] = { 1,  .2, 0},
	[2] = { 1,  .4, 0},
	[3] = { 1,  .6, 0},
	[4] = {.8,  .8, 0},
	[5] = { 0,   1, 0},
}

local YM = YandirsMight
local NAME = YM.name
local EM = EVENT_MANAGER
local SV

local inCombat = false

local ymFragment
local ymSet = 0 -- set equipped (0 - not equipped, 1 - non perfect, 2 - perfect)
local ymEnd = 0 -- when stagger effect ends (game seconds)
local ymStacks = 0  -- current number of stacks
local gmActive = false -- Giant's Might is active

local strformat = string.format

-- Check if at least 3 pieces of the set are equipped (2 more can be on the backbar during the check)
-- and returns 1 for non perfected, 2 for perfected, 0 if not equipped.
local function IsEquipped()
	local p, np = 0
	_, _, _, p = GetItemLinkSetInfo("|H1:item:162643:364:50:0:0:0:0:0:0:0:0:0:0:0:1:0:0:1:0:0:0|h|h", true)
	_, _, _, np = GetItemLinkSetInfo("|H1:item:162100:364:50:0:0:0:0:0:0:0:0:0:0:0:1:0:0:1:0:0:0|h|h", true)
	if p >= 3 then
		return 2
	elseif np >= 3 then
		return 1
	else
		return 0
	end
end

-- Get number of Giant's Endurance stacks on the player.
local function GetPlayerStacks()
	for i = 1, GetNumBuffs("player") do
		local _, _, _, _, stackCount, _, _, _, _, _, abilityId = GetUnitBuffInfo("player", i)
		if abilityId == GE_IDS[1] or abilityId == GE_IDS[2] then
			return stackCount
		end
	end
	return 0
end

local function Initialize()

	SV = ZO_SavedVars:New("YandirsMightSV", YM.varVersion, nil, YM.defaultSettings)

	YM.RestorePosition()

	-- Create UI fragment.
	ymFragment = ZO_SimpleSceneFragment:New(YandirsMightControl)
	ymFragment:SetConditional(function() return ymSet > 0 and inCombat or not YM.uiLocked end)
	HUD_SCENE:AddFragment(ymFragment)
	HUD_UI_SCENE:AddFragment(ymFragment)

	-- Update YandirsMightControl.
	local function UpdateStacks(stackCount)
		local r, g, b = 1, 0, 0
		if STATES[stackCount] then
			r, g, b = unpack(STATES[stackCount])
		end
		YandirsMightControl_BG:SetColor(r, g, b)
		if not gmActive then
			YandirsMightControl_Num:SetText(stackCount)
			YandirsMightControl_Icon:SetColor(1, 0, 0)
		end
	end

	local function CombatState()
		ymSet = IsEquipped()
		inCombat = IsUnitInCombat("player")
		ymFragment:Refresh()
	end

	local function PlayerActivated()
		CombatState()
		UpdateStacks(GetPlayerStacks())
	end

	-- Number of Giant's Endurance stacks changed.
	local function OnStackChanged(_, changeType, _, _, _, _, endTime, stackCount, _, _, _, _, _, _, unitId, abilityId)
		-- This check is needed in case the player has both sets on. We only track one of them.
		if abilityId == GE_IDS[ymSet] then
			UpdateStacks(stackCount)
		end
	end

	-- Giant's Might end time.
	local gmEnd = 0

	-- Giant's Might buff update.
	local function GMTick()
		local remain = gmEnd - GetFrameTimeSeconds()
		if remain >= 0 then
			gmActive = true
			YandirsMightControl_Num:SetText(strformat('%0.1f', remain))
		else
			gmActive = false
			EVENT_MANAGER:UnregisterForUpdate(NAME)
			YandirsMightControl_Icon:SetTexture(GetAbilityIcon(GE_IDS[1]))
			UpdateStacks(0)
		end
	end

	-- Giant's Might buff gained.
	local function OnEffectGained(_, changeType, _, _, _, _, endTime, stackCount, _, _, _, _, _, _, unitId, abilityId)
		-- This check is needed in case the player has both sets on. We only track one of them.
		if abilityId == GM_IDS[ymSet] then
			gmEnd = endTime
			gmActive = true

			EVENT_MANAGER:UnregisterForUpdate(NAME)
			EVENT_MANAGER:RegisterForUpdate(NAME, 100, GMTick)

			YandirsMightControl_Icon:SetTexture(GetAbilityIcon(GM_IDS[1]))
			YandirsMightControl_Icon:SetColor(0, 1, 0)
		end
	end

	EM:RegisterForEvent(NAME, EVENT_PLAYER_ACTIVATED, PlayerActivated)
	EM:RegisterForEvent(NAME, EVENT_PLAYER_COMBAT_STATE, CombatState)

	for k, id in ipairs(GE_IDS) do
		EM:RegisterForEvent(NAME .. 'GE' .. k, EVENT_EFFECT_CHANGED, OnStackChanged)
		EM:AddFilterForEvent(NAME .. 'GE' .. k, EVENT_EFFECT_CHANGED, REGISTER_FILTER_ABILITY_ID, GE_IDS)
		EM:AddFilterForEvent(NAME .. 'GE' .. k, EVENT_EFFECT_CHANGED, REGISTER_FILTER_SOURCE_COMBAT_UNIT_TYPE, COMBAT_UNIT_TYPE_PLAYER)
	end

	for k, id in ipairs(GM_IDS) do
		EM:RegisterForEvent(NAME .. 'GM' .. k, EVENT_EFFECT_CHANGED, OnEffectGained)
		EM:AddFilterForEvent(NAME .. 'GM' .. k, EVENT_EFFECT_CHANGED, REGISTER_FILTER_ABILITY_ID, GM_IDS)
		EM:AddFilterForEvent(NAME .. 'GM' .. k, EVENT_EFFECT_CHANGED, REGISTER_FILTER_SOURCE_COMBAT_UNIT_TYPE, COMBAT_UNIT_TYPE_PLAYER)
		EM:AddFilterForEvent(NAME .. 'GM' .. k, EVENT_COMBAT_EVENT, REGISTER_FILTER_COMBAT_RESULT, ACTION_RESULT_EFFECT_GAINED)
	end

end

function YM.Move()

	SV.controlCenterX, SV.controlCenterY = YandirsMightControl:GetCenter()

	YandirsMightControl:ClearAnchors()
	YandirsMightControl:SetAnchor(CENTER, GuiRoot, TOPLEFT, SV.controlCenterX, SV.controlCenterY)

end

function YM.RestorePosition()

	local controlCenterX = SV.controlCenterX
	local controlCenterY = SV.controlCenterY

	if controlCenterX or controlCenterY then
		YandirsMightControl:ClearAnchors()
		YandirsMightControl:SetAnchor(CENTER, GuiRoot, TOPLEFT, controlCenterX, controlCenterY)
	end

	YandirsMightControl_Icon:SetTexture(GetAbilityIcon(GE_IDS[1]))

end

local function OnAddOnLoaded(event, addonName)
	if addonName == NAME then
		EM:UnregisterForEvent(NAME, EVENT_ADD_ON_LOADED)
		Initialize()
	end
end

EM:RegisterForEvent(NAME, EVENT_ADD_ON_LOADED, OnAddOnLoaded)

SLASH_COMMANDS["/ym"] = function(str)
	YM.uiLocked = not YM.uiLocked
	ymFragment:Refresh()
end